import javax.swing.JFrame;
import javax.swing.SwingUtilities;

import com.mindfusion.common.DateTime;
import com.mindfusion.drawing.Colors;
import com.mindfusion.drawing.SolidBrush;
import com.mindfusion.spreadsheet.Cell;
import com.mindfusion.spreadsheet.CellChangedEvent;
import com.mindfusion.spreadsheet.CellRange;
import com.mindfusion.spreadsheet.CellValidationEvent;
import com.mindfusion.spreadsheet.HorizontalAlignment;
import com.mindfusion.spreadsheet.IStyle;
import com.mindfusion.spreadsheet.InplaceEditValidationEvent;
import com.mindfusion.spreadsheet.Measure;
import com.mindfusion.spreadsheet.VerticalAlignment;
import com.mindfusion.spreadsheet.Workbook;
import com.mindfusion.spreadsheet.WorkbookAdapter;
import com.mindfusion.spreadsheet.WorkbookView;
import com.mindfusion.spreadsheet.WorkbookViewAdapter;
import com.mindfusion.spreadsheet.Worksheet;


public class InvoiceTemplate
{
	// TODO: add product button
	public static void main(String[] args)
	{
		SwingUtilities.invokeLater(new Runnable()
		{
			public void run()
			{
				MainWindow window = new MainWindow();
				window.setTitle("JSpreadsheet Sample: Invoice Template");
				window.setVisible(true);
			}
		});
	}
	
	public static class MainWindow extends JFrame
	{
		public MainWindow()
		{
			setDefaultCloseOperation(EXIT_ON_CLOSE);
			setSize(1280, 1024);

			workbook = new Workbook();
			Worksheet activeSheet = workbook.getWorksheets().add("Invoice");

			view = new WorkbookView();
			view.setWorkbook(workbook);
			add(view);

			view.setShowGridLines(false);
			view.setAllowAutoFill(false);
			view.setShowTabs(false);
			view.setAllowMoveCells(false);
			view.setAllowMoveHeaders(false);

			activeSheet.beginInit();

			IStyle globalStyle = activeSheet.getCellRanges().get(0, 0, activeSheet.getColumns().getCount() - 1, activeSheet.getRows().getCount() - 1).getStyle();
			globalStyle.setFontName("Tahoma");
			globalStyle.setFontSize(12.0);

			CellRange heading = activeSheet.getCellRanges().get("A1:G1");
			heading.merge();
			activeSheet.getCells().get("A1").setData("Invoice");
			IStyle headingStyle = heading.getStyle();
			headingStyle.setFontSize(14.0);
			headingStyle.setFontBold(true);
			headingStyle.setHorizontalAlignment(HorizontalAlignment.Right);
			headingStyle.setVerticalAlignment(VerticalAlignment.Middle);
			headingStyle.setPaddingRight(new Measure("5pt"));
			headingStyle.setTextColor(Colors.DarkSlateBlue);
			headingStyle.setBorderBottomColor(Colors.DarkSlateBlue);
			headingStyle.setBorderBottomSize(new Measure("3pt"));

			CellRange address = activeSheet.getCellRanges().get("A3:C3");
			address.merge();
			activeSheet.getCells().get("A3").setData("(address)");
			IStyle addressStyle = address.getStyle();
			addressStyle.setTextColor(Colors.Gray);
			addressStyle.setVerticalAlignment(VerticalAlignment.Middle);
			addressStyle.setFontItalic(true);
			addressStyle.setFontSize(7.5);

			Cell date = activeSheet.getCells().get("F3");
			date.setData("Date:");
			IStyle dateStyle = date.getStyle();
			dateStyle.setFontBold(true);
			dateStyle.setTextColor(Colors.DarkSlateBlue);
			dateStyle.setVerticalAlignment(VerticalAlignment.Middle);
			dateStyle.setHorizontalAlignment(HorizontalAlignment.Right);
			date = activeSheet.getCells().get("G3");
			date.setData(DateTime.now());
			dateStyle = date.getStyle();
			dateStyle.setHorizontalAlignment(HorizontalAlignment.Left);
			dateStyle.setVerticalAlignment(VerticalAlignment.Middle);
			dateStyle.setFormat("dd-MMM-yy");

			Cell shipTo = activeSheet.getCells().get("A5");
			shipTo.setData("Ship to:");
			IStyle shipToStyle = shipTo.getStyle();
			shipToStyle.setFontBold(true);
			shipToStyle.setTextColor(Colors.DarkSlateBlue);
			shipToStyle.setVerticalAlignment(VerticalAlignment.Middle);
			shipToStyle.setHorizontalAlignment(HorizontalAlignment.Right);

			activeSheet.getCellRanges().get("B5:C5").merge();
			activeSheet.getCells().get("B5").setData("(ship name)");
			activeSheet.getCellRanges().get("B6:C6").merge();
			activeSheet.getCells().get("B6").setData("(ship address)");
			activeSheet.getCellRanges().get("B7:C7").merge();
			activeSheet.getCells().get("B7").setData("(ship region)");
			activeSheet.getCellRanges().get("B8:C8").merge();
			activeSheet.getCells().get("B8").setData("(ship country)");
			shipToStyle = activeSheet.getCellRanges().get("B5:C8").getStyle();
			shipToStyle.setVerticalAlignment(VerticalAlignment.Middle);
			shipToStyle.setTextColor(Colors.Gray);

			Cell billTo = activeSheet.getCells().get("D5");
			billTo.setData("Bill to:");
			IStyle billToStyle = billTo.getStyle();
			billToStyle.setFontBold(true);
			billToStyle.setTextColor(Colors.DarkSlateBlue);
			billToStyle.setVerticalAlignment(VerticalAlignment.Middle);
			billToStyle.setHorizontalAlignment(HorizontalAlignment.Right);

			activeSheet.getCellRanges().get("E5:F5").merge();
			activeSheet.getCells().get("E5").setData("(bill name)");
			activeSheet.getCellRanges().get("E6:F6").merge();
			activeSheet.getCells().get("E6").setData("(bill address)");
			activeSheet.getCellRanges().get("E7:F7").merge();
			activeSheet.getCells().get("E7").setData("(bill region)");
			activeSheet.getCellRanges().get("E8:F8").merge();
			activeSheet.getCells().get("E8").setData("(bill country)");
			shipToStyle = activeSheet.getCellRanges().get("E5:F8").getStyle();
			shipToStyle.setVerticalAlignment(VerticalAlignment.Middle);
			shipToStyle.setTextColor(Colors.Gray);

			String[] orderTexts = new String[] { "Order ID", "Customer ID", "Employee ID",
				"Order Date", "Required Date", "Shipped Date", "Ship Via" };
			for (int i = 0; i < orderTexts.length; i++)
				activeSheet.getCells().get(i, 9).setData(orderTexts[i]);
			IStyle orderHeadingStyle = activeSheet.getCellRanges().get(0, 9, 6, 9).getStyle();
			orderHeadingStyle.setBorderLeftColor(Colors.DarkBlue);
			orderHeadingStyle.setBorderTopColor(Colors.DarkBlue);
			orderHeadingStyle.setBorderRightColor(Colors.DarkBlue);
			orderHeadingStyle.setBorderBottomColor(Colors.DarkBlue);
			orderHeadingStyle.setBackground(new SolidBrush(Colors.DarkBlue));
			orderHeadingStyle.setTextColor(Colors.White);
			orderHeadingStyle.setFontBold(true);
			orderHeadingStyle.setHorizontalAlignment(HorizontalAlignment.Center);
			orderHeadingStyle.setVerticalAlignment(VerticalAlignment.Bottom);

			String[] orderPlaceholders = new String[] { "(order id)", "(customer id)", "(employee id)",
				"(order date)", "(required date)", "(shipped date)", "(ship via)" };
			for (int i = 0; i < orderPlaceholders.length; i++)
				activeSheet.getCells().get(i, 10).setData(orderPlaceholders[i]);
			IStyle orderContentStyle = activeSheet.getCellRanges().get(0, 10, 6, 10).getStyle();
			orderContentStyle.setBorderLeftColor(Colors.DarkBlue);
			orderContentStyle.setBorderTopColor(Colors.DarkBlue);
			orderContentStyle.setBorderRightColor(Colors.DarkBlue);
			orderContentStyle.setBorderBottomColor(Colors.DarkBlue);
			orderContentStyle.setTextColor(Colors.Gray);
			orderContentStyle.setHorizontalAlignment(HorizontalAlignment.Center);
			orderContentStyle.setVerticalAlignment(VerticalAlignment.Middle);
			activeSheet.getCellRanges().get("D11:F11").getStyle().setFormat("dd-MMM-yy");

			String[] productTexts = new String[] { "Product ID", "Product Name", "Product Name",
				"Quantity", "Unit Price", "Discount", "Extended Price" };
			for (int i = 0; i < productTexts.length; i++)
				activeSheet.getCells().get(i, 12).setData(productTexts[i]);
			IStyle productHeadingStyle = activeSheet.getCellRanges().get(0, 12, 6, 12).getStyle();
			productHeadingStyle.setBorderTopColor(Colors.DarkBlue);
			productHeadingStyle.setBorderBottomColor(Colors.DarkBlue);
			productHeadingStyle.setBackground(new SolidBrush(Colors.DarkBlue));
			productHeadingStyle.setTextColor(Colors.White);
			productHeadingStyle.setFontBold(true);
			productHeadingStyle.setHorizontalAlignment(HorizontalAlignment.Left);
			productHeadingStyle.setVerticalAlignment(VerticalAlignment.Bottom);
			activeSheet.getCellRanges().get(1, 12, 2, 12).merge();

			addProduct();

			Cell total = activeSheet.getCells().get("F15");
			total.setData("Total:");
			IStyle totalStyle = total.getStyle();
			totalStyle.setFontBold(true);
			totalStyle.setTextColor(Colors.DarkSlateBlue);
			totalStyle.setVerticalAlignment(VerticalAlignment.Middle);
			totalStyle.setHorizontalAlignment(HorizontalAlignment.Right);
			total = activeSheet.getCells().get("G15");
			total.setData("=SUM(G14)");
			totalStyle = total.getStyle();
			totalStyle.setHorizontalAlignment(HorizontalAlignment.Left);
			totalStyle.setVerticalAlignment(VerticalAlignment.Middle);
			totalStyle.setFormat("$#,##0.00;($#,##0.00)");

			activeSheet.getRows().get(0).setHeight(new Measure("48pt"));
			activeSheet.getRows().get(1).setHeight(new Measure("5pt"));
			activeSheet.getRows().get(2).setHeight(new Measure("32pt"));
			activeSheet.getRows().get(3).setHeight(new Measure("5pt"));
			activeSheet.getRows().get(8).setHeight(new Measure("20pt"));
			activeSheet.getRows().get(9).setHeight(new Measure("20pt"));
			activeSheet.getRows().get(10).setHeight(new Measure("18pt"));
			activeSheet.getRows().get(12).setHeight(new Measure("20pt"));
			activeSheet.getRows().get(13, 14).setHeight(new Measure("18pt"));
			activeSheet.getColumns().get(0, 6).setWidth(new Measure("72pt"));

			activeSheet.endInit();

			workbook.addWorkbookListener(new WorkbookAdapter()
			{
				@Override
				public void worksheetCellChanging(CellValidationEvent e)
				{
					e.setCancel(!canEdit(e.getCell()));
				}

				@Override
				public void worksheetCellChanged(CellChangedEvent e)
				{
					e.getCell().getStyle().setTextColor(Colors.Black);
				}
			});
			view.addViewListener(new WorkbookViewAdapter()
			{
				@Override
				public void inplaceEditStarting(InplaceEditValidationEvent e)
				{
					if (e.getItem() instanceof Cell)
					{
						Cell cell = (Cell)e.getItem();
						e.setCancel(!canEdit(cell));
					}
				}
			});
		}

		private boolean canEdit(Cell cell)
		{
			int c = cell.getColumn();
			int r = cell.getRow();

			if (r == 2 && c == 0 ||
				r == 2 && c == 6 ||
				r == 10 && c <= 6 ||
				r >= 13 && r <= 12 + products && c <= 5 ||
				c == 1 && r >= 4 && r <= 7 ||
				c == 4 && r >= 4 && r <= 7)
			{
				return true;
			}

			return false;
		}

		private void addProduct()
		{
			Worksheet activeSheet = view.getActiveWorksheet();

			activeSheet.beginInit();
			String[] productPlaceholders = new String[] { "(product id)", "(product name)", "(product name)",
				"(quantity)", "(unit price)", "(discount)", };
			for (int i = 0; i < productPlaceholders.length; i++)
				activeSheet.getCells().get(i, 12 + products).setData(productPlaceholders[i]);
			activeSheet.getCells().get(6, 12 + products).setData("=IFERROR(D14*E14*(1-F14), 0)", 0, products - 1);
			IStyle productContentStyle = activeSheet.getCellRanges().get(0, 12 + products, 6, 12 + products).getStyle();
			productContentStyle.setBorderBottomColor(Colors.DarkBlue);
			productContentStyle.setHorizontalAlignment(HorizontalAlignment.Left);
			productContentStyle.setVerticalAlignment(VerticalAlignment.Middle);
			productContentStyle = activeSheet.getCellRanges().get(0, 12 + products, 5, 12 + products).getStyle();
			productContentStyle.setTextColor(Colors.Gray);
			activeSheet.getCellRanges().get(1, 12 + products, 2, 12 + products).merge();
			activeSheet.getCells().get(4, 12 + products).getStyle().setFormat("$#,##0.00;($#,##0.00)");
			activeSheet.getCells().get(5, 12 + products).getStyle().setFormat(workbook.getDefaultNumericFormats().getPercentFormats().get(0).getFormat());
			activeSheet.getCells().get(6, 12 + products).getStyle().setFormat("$#,##0.00;($#,##0.00)");
			activeSheet.endInit();
		}


		private static final long serialVersionUID = 1L;

		private Workbook workbook;
		private WorkbookView view;
		private int products = 1;
	}
}